/***************************************************************************
                                  Geometry.C 
                             -------------------
    begin                : 2001
    copyright            : (C)2001-2003 Multi-Agent Systems Laboratory,
                           University of Science and Technology of China 
                           Zhanxiang Huang, Yang Yang and Xiaoping Chen.
    email                : huangzx@mail.ustc.edu.cn
 ***************************************************************************/

#include "Geometry.h"


/*! This function returns the Sign of a give double.
    1 Is positive, -1 Is negative
    \param d1 first parameter
    \return the Sign of this double */
int Sign( double d1 )
{
  return (d1>0)?1:-1;
}

/*! This function returns the Maximum of two given doubles.
    \param d1 first parameter
    \param d2 second parameter
    \return the Maximum of these two parameters */
double Max( double d1, double d2 )
{
  return (d1>d2)?d1:d2;
}

/*! This function returns the Minimum of two given doubles.
    \param d1 first parameter
    \param d2 second parameter
    \return the Minimum of these two parameters */
double Min( double d1, double d2 )
{
  return (d1<d2)?d1:d2;
}


/*! This function converts an angle in radians to the corresponding angle in
    degrees.
    \param x an angle in radians
    \return the corresponding angle in degrees */
AngDeg Rad2Deg( AngRad x )
{
  return ( x * 180 / M_PI );
}

/*! This function converts an angle in degrees to the corresponding angle in
    radians.
    \param x an angle in degrees
    \return the corresponding angle in radians */
AngRad Deg2Rad( AngDeg x )
{
  return ( x * M_PI / 180 );
}

/*! This function returns the cosine of a given angle in degrees using the
    built-in cosine function that works with angles in radians.
    \param x an angle in degrees
    \return the cosine of the given angle */
double CosDeg( AngDeg x )
{
  return ( cos( Deg2Rad( x ) ) );
}

/*! This function returns the sine of a given angle in degrees using the
    built-in sine function that works with angles in radians.
    \param x an angle in degrees
    \return the sine of the given angle */
double SinDeg( AngDeg x )
{
  return ( sin( Deg2Rad( x ) ) );
}

/*! This function returns the tangent of a given angle in degrees using the
    built-in tangent function that works with angles in radians.
    \param x an angle in degrees
    \return the tangent of the given angle */
double TanDeg( AngDeg x )
{
  return ( tan( Deg2Rad( x ) ) );
}

/*! This function returns the principal value of the arc tangent of x in degrees
    using the built-in arc tangent function which returns this value in radians.
    \param x a double value
    \return the arc tangent of the given value in degrees */
AngDeg ATanDeg( double x )
{
  return ( Rad2Deg( atan( x ) ) );
}

/*! This function returns the principal value of the arc tangent of y/x in
    degrees using the Signs of both arguments to deterMine the quadrant of the
    return value. For this the built-in 'atan2' function Is used which returns
    this value in radians.
    \param x a double value
    \param y a double value
    \return the arc tangent of y/x in degrees taking the Signs of x and y into
    account */
double Atan2Deg( double x, double y )
{
  if( fabs( x ) < EPSILON && fabs( y ) < EPSILON )
    return ( 0.0 );

  return ( Rad2Deg( atan2( x, y ) ) );
}

/*! This function returns the principal value of the arc cosine of x in degrees
    using the built-in arc cosine function which returns this value in radians.
    \param x a double value
    \return the arc cosine of the given value in degrees */
AngDeg ACosDeg( double x )
{
  if( x >= 1 )
    return ( 0.0 );
  else if( x <= -1 )
    return ( 180.0 );

  return ( Rad2Deg( acos( x ) ) );
}

/*! This function returns the principal value of the arc sine of x in degrees
    using the built-in arc sine function which returns this value in radians.
    \param x a double value
    \return the arc sine of the given value in degrees */
AngDeg ASinDeg( double x )
{
  if( x >= 1 )
    return ( 90.0 );
  else if ( x <= -1 )
    return ( -90.0 );

  return ( Rad2Deg( asin( x ) ) );
}




// matrix functions
void    MatrixMul15_51(double s1[5],double s2[5],double *d)
{
    (*d) = 0;
    for(int i=0;i<5;i++)
        (*d) += s1[i]*s2[i];
}

void    MatrixMul15_55(double s1[5],double s2[5][5],double (*d)[5])
{
    double temp[5];
    for(int i=0;i<5;i++)
    {
        temp[i] = 0;
        for(int j=0;j<5;j++)
            temp[i] += s1[j]*s2[j][i];
    }
    for(int i=0;i<5;i++) (*d)[i] = temp[i];
}

void    MatrixMul51_1(double s1[5],double s2,double (*d)[5])
{
    double temp[5];
    for(int i=0;i<5;i++)
        temp[i] = s1[i]*s2;
    for(int i=0;i<5;i++)
        (*d)[i] = temp[i];
}

void    MatrixDiv51_1(double s1[5],double s2,double (*d)[5])
{
    double temp[5];
    for(int i=0;i<5;i++)
        temp[i] = s1[i]/s2;
    for(int i=0;i<5;i++)
        (*d)[i] = temp[i];
}

void    MatrixMul51_15(double s1[5],double s2[5],double (*d)[5][5])
{
    double temp[5][5];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            temp[i][j] = s1[i]*s2[j];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            (*d)[i][j] = temp[i][j];
}

void    MatrixDiv55_1(double s1[5][5],double s2,double (*d)[5][5])
{
    double temp[5][5];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            temp[i][j] = s1[i][j]/s2;
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            (*d)[i][j] = temp[i][j];
}

void    MatrixMul55_51(double s1[5][5],double s2[5],double (*d)[5])
{
    double temp[5];
    for(int i=0;i<5;i++)
    {
        temp[i] = 0;
        for(int j=0;j<5;j++)
            temp[i] += s1[i][j]*s2[j];
    }
    for(int i=0;i<5;i++) (*d)[i] = temp[i];
}

void    MatrixMul55_55(double s1[5][5],double s2[5][5],double (*d)[5][5])
{
    double temp[5][5];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
        {
            temp[i][j] = 0;
            for(int k=0;k<5;k++)
                temp[i][j] += s1[i][k]*s2[k][j];
        }
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            (*d)[i][j] = temp[i][j];
}

void    MatrixSub55(double s1[5][5],double s2[5][5],double (*d)[5][5])
{
    double temp[5][5];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            temp[i][j] = s1[i][j] - s2[i][j];
    for(int i=0;i<5;i++)
        for(int j=0;j<5;j++)
            (*d)[i][j] = temp[i][j];
}

void    MatrixAdd51_51(double s1[5],double s2[5],double (*d)[5])
{
    double temp[5];
    for(int i=0;i<5;i++)
        temp[i] = s1[i]+s2[i];
    for(int i=0;i<5;i++)
        (*d)[i] = temp[i];
}












_Vect::_Vect( double x, double y )
{
  SetVect( x, y);
}

_Vect::_Vect(const  _Vect &v)
{
        m_x = v.GetX();
        m_y = v.GetY();        
}

// Overloaded version of unary Minus operator for _Vects. It returns the
//    negative _Vect, i.e. both the x- and y-coordinates are multiplied by
//   -1. The current _Vect itself Is left unchanged.
//    \return a negated version of the current _Vect
_Vect _Vect::operator - ( )
{
  return ( _Vect( -m_x, -m_y ) );
}

// Overloaded version of the binary plus operator for adding a given double
//    value to a _Vect. The double value Is added to both the x- and
//    y-coordinates of the current _Vect. The current _Vect itself Is
//    left unchanged.
//    \param d a double value which has to be added to both the x- and
//    y-coordinates of the current _Vect
//    \return the result of adding the given double value to the current
//    _Vect */
_Vect _Vect::operator + ( const double &d )
{
  return ( _Vect( m_x + d, m_y + d ) );
}

// Overloaded version of the binary plus operator for _Vects. It returns
//    the sum of the current _Vect and the given _Vect by adding their
//    x- and y-coordinates. The _Vects themselves are left unchanged.
//    \param p a _Vect
//    \return the sum of the current _Vect and the given _Vect
_Vect _Vect::operator + ( const _Vect &p )
{
  return ( _Vect( m_x + p.m_x, m_y + p.m_y ) );
}

// Overloaded version of the binary Minus operator for subtracting a given
//    double value from a _Vect. The double value Is subtracted from both
//    the x- and y-coordinates of the current _Vect. The current _Vect
//    itself Is left unchanged.
//    \param d a double value which has to be subtracted from both the x- and
//    y-coordinates of the current _Vect
//    \return the result of subtracting the given double value from the current
//    _Vect 
_Vect _Vect::operator - ( const double &d )
{
  return ( _Vect( m_x - d, m_y - d ) );
}

// Overloaded version of the binary Minus operator for _Vects. It returns
//    the difference between the current _Vect and the given _Vect by
//    subtracting their x- and y-coordinates. The _Vects themselves are left
//    unchanged.
//    \param p a _Vect
//    \return the difference between the current _Vect and the given
//    _Vect 
_Vect _Vect::operator - ( const _Vect &p )
{
  return ( _Vect( m_x - p.m_x, m_y - p.m_y ) );
}

// Overloaded version of the multiplication operator for multiplying a
//    _Vect by a given double value. Both the x- and y-coordinates of the
//    current _Vect are multiplied by this value. The current _Vect
//    itself Is left unchanged.
//    \param d the multiplication factor
//    \return the result of multiplying the current _Vect by the given
//    double value 
_Vect _Vect::operator * ( const double &d  )
{
  return ( _Vect( m_x * d, m_y * d  ) );
}

// Overloaded version of the multiplication operator for _Vects. It
//    returns the product of the current _Vect and the given _Vect by
//    multiplying their x- and y-coordinates. The _Vects themselves are left
//    unchanged.
//    \param p a _Vect
 //   \return the product of the current _Vect and the given _Vect 
_Vect _Vect::operator * ( const _Vect &p )
{
  return ( _Vect( m_x * p.m_x, m_y * p.m_y ) );
}

// Overloaded version of the division operator for dividing a _Vect by a
//    given double value. Both the x- and y-coordinates of the current _Vect
//    are divided by this value. The current _Vect itself Is left unchanged.
//    \param d the division factor
//    \return the result of dividing the current _Vect by the given double
//    value
_Vect _Vect::operator / ( const double &d )
{
  return ( _Vect( m_x / d, m_y / d  ) );
}

// Overloaded version of the division operator for _Vects. It returns the
//    quotient of the current _Vect and the given _Vect by dividing
//    their x- and y-coordinates. The _Vects themselves are left unchanged.
//    \param p a _Vect
//    \return the quotient of the current _Vect and the given _Vect 
_Vect _Vect::operator / ( const _Vect &p )
{
  return ( _Vect( m_x / p.m_x, m_y / p.m_y ) );
}

// Overloaded version of the asSignment operator for asSigning a given double
//    value to both the x- and y-coordinates of the current _Vect. This
//    changes the current _Vect itself.
//    \param d a double value which has to be asSigned to both the x- and
//    y-coordinates of the current _Vect */
void _Vect::operator = ( const double &d )
{
  m_x = d;
  m_y = d;
}

void _Vect::operator = (const  _Vect &v)
{
        m_x = v.GetX();
        m_y = v.GetY();        
}


// Overloaded version of the sum-asSignment operator for _Vects. It
//    returns the sum of the current _Vect and the given _Vect by
//    adding their x- and y-coordinates. This changes the current _Vect
//    itself.
//    \param p a _Vect which has to be added to the current _Vect 
void _Vect::operator +=( const _Vect &p )
{
  m_x += p.m_x;
  m_y += p.m_y;
}

// Overloaded version of the sum-asSignment operator for adding a given double
//   value to a _Vect. The double value Is added to both the x- and
//    y-coordinates of the current _Vect. This changes the current
//    _Vect itself.
//    \param d a double value which has to be added to both the x- and
//    y-coordinates of the current _Vect 
void _Vect::operator += ( const double &d )
{
  m_x += d;
  m_y += d;
}

// Overloaded version of the difference-asSignment operator for _Vects.
//    It returns the difference between the current _Vect and the given
//    _Vect by subtracting their x- and y-coordinates. This changes the
//    current _Vect itself.
//    \param p a _Vect which has to be subtracted from the current
//    _Vect 
void _Vect::operator -=( const _Vect &p )
{
  m_x -= p.m_x;
  m_y -= p.m_y;
}

// Overloaded version of the difference-asSignment operator for subtracting a
//    given double value from a _Vect. The double value Is subtracted from
//   both the x- and y-coordinates of the current _Vect. This changes the
//    current _Vect itself.
//    \param d a double value which has to be subtracted from both the x- and
 //   y-coordinates of the current _Vect 
void _Vect::operator -=( const double &d )
{
  m_x -= d;
  m_y -= d;
}

// Overloaded version of the multiplication-asSignment operator for
//   _Vects. It returns the product of the current _Vect and the
//    given _Vect by multiplying their x- and y-coordinates. This changes
//    the current _Vect itself.
//    \param p a _Vect by which the current _Vect has to be
//    multiplied 
void _Vect::operator *=( const _Vect &p )
{
  m_x *= p.m_x;
  m_y *= p.m_y;
}

// Overloaded version of the multiplication-asSignment operator for multiplying
//    a _Vect by a given double value. Both the x- and y-coordinates of the
//    current _Vect are multiplied by this value. This changes the current
//    _Vect itself.
//    \param d a double value by which both the x- and y-coordinates of the
//    current _Vect have to be multiplied 
void _Vect::operator *=( const double &d )
{
  m_x *= d;
  m_y *= d;
}

// Overloaded version of the division-asSignment operator for _Vects. It
//    returns the quotient of the current _Vect and the given _Vect by
//    dividing their x- and y-coordinates. This changes the current _Vect
//    itself.
 //   \param p a _Vect by which the current _Vect has to be divided 
void _Vect::operator /=( const _Vect &p )
{
  m_x /= p.m_x;
  m_y /= p.m_y;
}

// Overloaded version of the division-asSignment operator for dividing a
//    _Vect by a given double value. Both the x- and y-coordinates of the
//    current _Vect are divided by this value. This changes the current
//    _Vect itself.
//    \param d a double value by which both the x- and y-coordinates of the
//    current _Vect have to be divided 
void _Vect::operator /=( const double &d )
{
  m_x /= d;
  m_y /= d;
}

// Overloaded version of the inequality operator for _Vects. It
//    deterMines whether the current _Vect Is unequal to the given
//    _Vect by comparing their x- and y-coordinates.
//    \param p a _Vect
//    \return true when either the x- or y-coordinates of the given _Vect
 //   and the current _Vect are different; false otherwise 
bool _Vect::operator !=( const _Vect &p )
{
  return ( ( m_x != p.m_x ) || ( m_y != p.m_y ) );
}

// Overloaded version of the inequality operator for comparing a _Vect to
//    a double value. It deterMines whether either the x- or y-coordinate of the
//   current _Vect Is unequal to the given double value.
//    \param d a double value with which both the x- and y-coordinates of the
//    current _Vect have to be compared.
//    \return true when either the x- or y-coordinate of the current _Vect
//    Is unequal to the given double value; false otherwise 
bool _Vect::operator !=( const double &d )
{
  return ( ( m_x != d ) || ( m_y != d ) );
}

// Overloaded version of the equality operator for _Vects. It deterMines
//    whether the current _Vect Is equal to the given _Vect by
//    comparing their x- and y-coordinates.
//    \param p a _Vect
//    \return true when both the x- and y-coordinates of the given _Vect and
//    the current _Vect are equal; false otherwise 
bool _Vect::operator ==( const _Vect &p )
{
  return ( ( m_x == p.m_x ) && ( m_y == p.m_y ) );
}

// Overloaded version of the equality operator for comparing a _Vect to a
//    double value. It deterMines whether both the x- and y-coordinates of the
//    current _Vect are equal to the given double value.
//    \param d a double value with which both the x- and y-coordinates of the
//    current _Vect have to be compared.
//    \return true when both the x- and y-coordinates of the current _Vect
//    are equal to the given double value; false otherwise 
bool _Vect::operator ==( const double &d )
{
  return ( ( m_x == d ) && ( m_y == d ) );
}

// Overloaded version of the C++ output operator for _Vects. This
//   operator makes it possible to use _Vects in output statements (e.g.
//    cout << v). The x- and y-coordinates of the _Vect are printed in the
//    format (x,y).
//    \param os output stream to which information should be written
//    \param v a _Vect which must be printed
//    \return output stream containing (x,y) 
ostream& operator <<( ostream &os, const  _Vect  &v )
{
        return ( os << "( " << v.GetX() << ", " << v.GetY() << " )" );
}

// This method writes the current _Vect to standard output. It can also
//   print a polar representation of the current _Vect.
//    \param cs a CoordSystemtT indicating whether a POLAR or CARTESIAN
//     representation of the current _Vect should be printed 
void _Vect::show( )
{
     cout << *this << endl;
}

// This method writes the current _Vect to a string. It can also write a
//   polar representation of the current _Vect.
//   \param cs a CoordSystemtT indicating whether a POLAR or CARTESIAN
//     representation of the current _Vect should be written
//    \return a string containing a polar or Cartesian representation of the
//    current _Vect depending on the value of the boolean argument 
string _Vect::get_str( )
{
    char buf[ 1024 ];

    sprintf( buf, "( %f, %f )", GetX( ), GetY( ) );
    string str( buf );
    return ( str );
}

// Set method for the x-coordinate of the current _Vect.
//    \param dX a double value representing a new x-coordinate
//    \return a boolean indicating whether the Update was successful 
bool _Vect::SetX( double dX )
{
  m_x = dX;
  return ( true );
}

// Get method for the x-coordinate of the current _Vect.
//    \return the x-coordinate of the current _Vect 
double _Vect::GetX( ) const
{
  return ( m_x );
}

// Set method for the y-coordinate of the current _Vect.
//    \param dY a double value representing a new y-coordinate
//    \return a boolean indicating whether the Update was successful 
bool _Vect::SetY( double dY )
{
  m_y = dY;
  return ( true );
}

// Get method for the y-coordinate of the current _Vect.
//    \return the y-coordinate of the current _Vect 
double _Vect::GetY( ) const
{
  return ( m_y );
}

// This method (re)Sets the coordinates of the current _Vect. The given
//   coordinates can either be polar or Cartesian coordinates. This Is indicated
//    by the value of the third argument.
//    \param dX a double value indicating either a new Cartesian x-coordinate when
//          cs=CARTESIAN or a new polar r-coordinate (distance) when cs=POLAR
//    \param dY a double value indicating either a new Cartesian y-coordinate when
//          cs=CARTESIAN or a new polar phi-coordinate (angle) when cs=POLAR
//    \param cs a CoordSystemT indicating whether x and y denote cartesian 
//    coordinates or polar coordinates 
void _Vect::SetVect( double dX = 0, double dY = 0)
{
    m_x = dX;
    m_y = dY;
}

/*! This method adjusts the coordinates of the current _Vect in such a way
    that the magnitude of the corresponding vector equals the double value which
    Is supplied as an argument. It thus scales the vector to a given length by
    multiplying both the x- and y-coordinates by the quotient of the argument
    and the current magnitude. This changes the _Vect itself.
    \param d a double value representing a new magnitude
    \return the result of scaling the vector corresponding with the current
    _Vect to the given magnitude thus yielding a different _Vect */
_Vect  _Vect::SetMagnitude( double d )
{
  if( GetMagnitude( ) > EPSILON )
     ( *this ) *= ( d / GetMagnitude( ) );

  return ( *this );
}

/*! This method deterMines the magnitude (length) of the vector corresponding
    with the current _Vect using the formula of Pythagoras.
    \return the length of the vector corresponding with the current
    _Vect */
double _Vect::GetMagnitude( ) const
{
  return ( sqrt( m_x * m_x + m_y * m_y ) );
}

/*! This method deterMines the direction of the vector corresponding with the
    current _Vect (the phi-coordinate in polar representation) using the
    arc tangent function. Note that the Signs of x and y have to be taken into
    account in order to deterMine the correct quadrant.
    \return the direction in degrees of the vector corresponding with the
    current _Vect */
AngDeg _Vect::GetDirection( ) const
{
  return ( Atan2Deg( m_y, m_x ) );
}



/*! This method deterMines the distance between the current VecPos and a
    given VecPos. This Is equal to the magnitude (length) of the vector
    connecting the two positions which Is the difference vector between them.
    \param p a VecPos
    \return the distance between the current VecPos and the given
    VecPos */
double VecPos::GetDistanceTo( const VecPos p )
{
  return ( ( *this - p ).GetMagnitude( ) );
}

/*! This method deterMines whether the current VecPos Is in front of a
    given VecPos, i.e. whether the x-coordinate of the current VecPos
    Is larger than the x-coordinate of the given VecPos.
    \param p a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is in front of the given
    VecPos; false otherwise */
bool VecPos::IsInFrontOf( const VecPos &p )
{
  return ( ( m_x > p.GetX( ) ) ? true : false );
}

/*! This method deterMines whether the x-coordinate of the current VecPos
    Is in front of (i.e. larger than) a given double value.
    \param d a double value to which the current x-coordinate must be compared
    \return true when the current x-coordinate Is in front of the given value;
    false otherwise */
bool VecPos::IsInFrontOf( const double &d )
{
  return ( ( m_x > d ) ? true : false );
}

/*! This method deterMines whether the current VecPos Is behind a given
    VecPos, i.e. whether the x-coordinate of the current VecPos Is
    smaller than the x-coordinate of the given VecPos.
    \param p a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is behind the given VecPos;
    false otherwise */
bool VecPos::IsBehindOf( const VecPos &p )
{
  return ( ( m_x < p.GetX( ) ) ? true : false );
}

/*! This method deterMines whether the x-coordinate of the current VecPos
    Is behind (i.e. smaller than) a given double value.
    \param d a double value to which the current x-coordinate must be compared
    \return true when the current x-coordinate Is behind the given value; false
    otherwise */
bool VecPos::IsBehindOf( const double &d )
{
  return ( ( m_x < d ) ? true : false );
}

/*! This method deterMines whether the current VecPos Is to the left of a
    given VecPos, i.e. whether the y-coordinate of the current VecPos
    Is smaller than the y-coordinate of the given VecPos.
    \param p a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is to the left of the given
    VecPos; false otherwise */
bool VecPos::IsLeftOf( const VecPos &p )
{
  return ( ( m_y < p.GetY( ) ) ? true : false );
}

/*! This method deterMines whether the y-coordinate of the current VecPos
    Is to the left of (i.e. smaller than) a given double value.
    \param d a double value to which the current y-coordinate must be compared
    \return true when the current y-coordinate Is to the left of the given
    value; false otherwise */
bool VecPos::IsLeftOf( const double &d )
{
  return ( ( m_y < d ) ? true : false );
}

/*! This method deterMines whether the current VecPos Is to the right of a
    given VecPos, i.e. whether the y-coordinate of the current VecPos
    Is larger than the y-coordinate of the given VecPos.
    \param p a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is to the right of the given
    VecPos; false otherwise */
bool VecPos::IsRightOf( const VecPos &p )
{
  return ( ( m_y > p.GetY( ) ) ? true : false );
}

/*! This method deterMines whether the y-coordinate of the current VecPos
    Is to the right of (i.e. larger than) a given double value.
    \param d a double value to which the current y-coordinate must be compared
    \return true when the current y-coordinate Is to the right of the given
    value; false otherwise */
bool VecPos::IsRightOf( const double &d )
{
  return ( ( m_y > d ) ? true : false );
}

/*! This method deterMines whether the current VecPos Is in between two
    given VecPoss when looking in the x-direction, i.e. whether the current
    VecPos Is in front of the first argument and behind the second.
    \param p1 a VecPos to which the current VecPos must be compared
    \param p2 a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is in between the two given
    VecPoss when looking in the x-direction; false otherwise */
bool VecPos::IsBetweenX( const VecPos &p1, const VecPos &p2 )
{
  return ( ( IsInFrontOf( p1 ) && IsBehindOf( p2 ) ) ? true : false );
}

/*! This method deterMines whether the x-coordinate of the current VecPos
    Is in between two given double values, i.e. whether the x-coordinate of the
    current VecPos Is in front of the first argument and behind the second.
    \param d1 a double value to which the current x-coordinate must be compared
    \param d2 a double value to which the current x-coordinate must be compared
    \return true when the current x-coordinate Is in between the two given
    values; false otherwise */
bool VecPos::IsBetweenX( const double &d1, const double &d2 )
{
  return ( ( IsInFrontOf( d1 ) && IsBehindOf( d2 ) ) ? true : false );
}

/*! This method deterMines whether the current VecPos Is in between two
    given VecPoss when looking in the y-direction, i.e. whether the current
    VecPos Is to the right of the first argument and to the left of the
    second.
    \param p1 a VecPos to which the current VecPos must be compared
    \param p2 a VecPos to which the current VecPos must be compared
    \return true when the current VecPos Is in between the two given
    VecPoss when looking in the y-direction; false otherwise */
bool VecPos::IsBetweenY( const VecPos &p1, const VecPos &p2 )
{
  return ( ( IsRightOf( p1 ) && IsLeftOf( p2 ) ) ? true : false );
}

/*! This method deterMines whether the y-coordinate of the current VecPos
    Is in between two given double values, i.e. whether the y-coordinate of the
    current VecPos Is to the right of the first argument and to the left
    of the second.
    \param d1 a double value to which the current y-coordinate must be compared
    \param d2 a double value to which the current y-coordinate must be compared
    \return true when the current y-coordinate Is in between the two given
    values; false otherwise */
bool VecPos::IsBetweenY( const double &d1, const double &d2 )
{
  return ( ( IsRightOf( d1 ) && IsLeftOf( d2 ) ) ? true : false );
}


